
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>

#include "nrf.h"
#include "nrf_drv_clock.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "nrf_drv_power.h"
#include "nrf_serial.h"
#include "app_timer.h"


#include "app_error.h"
#include "app_util.h"
#include "boards.h"



 //**************************************************************************************************
#define GSM_MAX_CHARS 128

void GSM_init(void);
void GSM_uninit(void);
ret_code_t GSM_send(void const * p_dataa, uint8_t len, uint32_t timeout);
ret_code_t GSM_tx_flush(uint32_t timeout_ms); 
ret_code_t GSM_read(char * p_dataa, uint8_t len, uint32_t timeout_ms);
ret_code_t GSM_abort_tx(void);  //also clears TX FIFO
ret_code_t GSM_drain_rx(void); //also Clears RX FIFO
void GSM_interrupt_enable(void);
void GSM_interrupt_disable(void);
void mqtt_connect(void);
void on_GSM_interrupt(void);
void off_GSM_interrupt(void);
void GSM_InterruptHandler(nrf_serial_t const* , nrf_serial_event_t );
unsigned char sendCommand_resp(char* ,unsigned int , char* , unsigned int , int );
void flushuart(void);
uint8_t send_sms(char* , char* , unsigned int );
uint8_t mqtt_publish(void);
  
 //**************************************************************************************************

#define pin_rx1 27//26//NRF_GPIO_PIN_MAP(0, 26)
#define pin_tx1 26//27//NRF_GPIO_PIN_MAP(0, 27)

#define OP_QUEUES_SIZE          3
#define APP_TIMER_PRESCALER     NRF_SERIAL_APP_TIMER_PRESCALER

static void sleep_handler(void)
{
    __WFE();
    __SEV();
    __WFE();
}

NRF_SERIAL_DRV_UART_CONFIG_DEF(m_uarte0_drv_config,
                      pin_rx1,pin_tx1,//RX_PIN_NUMBER, ARDUINO_SCL_PIN,
                      RTS_PIN_NUMBER, CTS_PIN_NUMBER,
                      NRF_UART_HWFC_DISABLED, NRF_UART_PARITY_EXCLUDED,
                      NRF_UART_BAUDRATE_115200,//NRF_UART_BAUDRATE_9600,//NRF_UART_BAUDRATE_115200,
                      UART_DEFAULT_CONFIG_IRQ_PRIORITY);

NRF_SERIAL_DRV_UART_CONFIG_DEF(m_uarte1_drv_config,
                      ARDUINO_SDA_PIN, TX_PIN_NUMBER,
                      RTS_PIN_NUMBER, CTS_PIN_NUMBER,
                      NRF_UART_HWFC_DISABLED, NRF_UART_PARITY_EXCLUDED,
                      NRF_UART_BAUDRATE_115200,
                      UART_DEFAULT_CONFIG_IRQ_PRIORITY);


#define SERIAL_FIFO_TX_SIZE 32
#define SERIAL_FIFO_RX_SIZE 32

NRF_SERIAL_QUEUES_DEF(serial0_queues, SERIAL_FIFO_TX_SIZE, SERIAL_FIFO_RX_SIZE);
NRF_SERIAL_QUEUES_DEF(serial1_queues, SERIAL_FIFO_TX_SIZE, SERIAL_FIFO_RX_SIZE);


#define SERIAL_BUFF_TX_SIZE 1
#define SERIAL_BUFF_RX_SIZE 1

NRF_SERIAL_BUFFERS_DEF(serial0_buffs, SERIAL_BUFF_TX_SIZE, SERIAL_BUFF_RX_SIZE);
NRF_SERIAL_BUFFERS_DEF(serial1_buffs, SERIAL_BUFF_TX_SIZE, SERIAL_BUFF_RX_SIZE);


NRF_SERIAL_CONFIG_DEF(serial0_config, NRF_SERIAL_MODE_DMA,
                      &serial0_queues, &serial0_buffs, GSM_InterruptHandler, sleep_handler);

NRF_SERIAL_CONFIG_DEF(serial2_config, NRF_SERIAL_MODE_DMA,
                      &serial0_queues, &serial0_buffs, NULL, sleep_handler);

NRF_SERIAL_CONFIG_DEF(serial1_config, NRF_SERIAL_MODE_DMA,
                      &serial1_queues, &serial1_buffs, NULL, sleep_handler);


NRF_SERIAL_UART_DEF(serial0_uarte, 0);
NRF_SERIAL_UART_DEF(serial1_uarte, 1);


//*******************************************************************************************************
   size_t written_size;
   size_t readed_size;
  //volatile bool tx_done;
  //volatile bool rx_done;

  volatile bool GSM_Received=0;
  volatile uint8_t GSM_Receive_count=0;

  volatile char rx_buff[256];
  volatile uint8_t rx_len;
  volatile uint8_t rx_index;
  bool resett=false;

  char arraow = '>';

   typedef struct
{
  unsigned char RxBuf[256];
  unsigned int RxBuf_Len;
  unsigned char TxBuf[256];
  unsigned int TxBuf_Len;

}uart_d;
  uart_d gsm_d;

//***********************************************************************************************************
uint32_t timee = 0;
static void log_init(void)
{
  ret_code_t err_code = NRF_LOG_INIT(NULL);
  APP_ERROR_CHECK(err_code);

  NRF_LOG_DEFAULT_BACKENDS_INIT();
}

int main(void)
{
    ret_code_t ret;

    log_init();

    NRF_LOG_INFO("PROGRAM STARTED");

    int32_t reset_reason = NRF_POWER->RESETREAS;
    NRF_LOG_INFO("Reset reason = 0x%08x.\n", reset_reason);
    NRF_POWER->RESETREAS = NRF_POWER->RESETREAS;

    ret = nrf_drv_clock_init();
    APP_ERROR_CHECK(ret);
    ret = nrf_drv_power_init(NULL);
    APP_ERROR_CHECK(ret);

    nrf_drv_clock_lfclk_request(NULL);
    ret = app_timer_init();
    APP_ERROR_CHECK(ret);

    // Initialize LEDs and buttons.
    bsp_board_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS);

   memset(&gsm_d,0,sizeof(gsm_d));

    ret = nrf_serial_init(&serial1_uarte, &m_uarte1_drv_config, &serial1_config);
    APP_ERROR_CHECK(ret);

    GSM_init();
//    mqtt_connect();
    //if(!send_sms("9652490222", "New Test2", strlen("New Test2")))
    //{
    //  int k=0;
    //}
    //else 
    //{
    //  int rr=0;
    //}

    if(!mqtt_publish())
    {
      int k=0;
      k++;
      memset(&gsm_d,0,sizeof(gsm_d));
    }
    else 
    {
      int rr=0;
      rr++;
      memset(&gsm_d,0,sizeof(gsm_d));
    }

    //static char tx_message[] = "Hello nrf_serial!\n\r";

    //ret = nrf_serial_write(&serial0_uarte,
    //                       tx_message,
    //                       strlen(tx_message),
    //                       NULL,
    //                       NRF_SERIAL_MAX_TIMEOUT);
    //(void)nrf_serial_flush(&serial0_uarte, 0);

    while (true)
    {
      timee = app_timer_cnt_get();
        //char c;
        //ret = nrf_serial_read(&serial0_uarte, &c, sizeof(c), NULL, 1000);
        //if (ret != NRF_SUCCESS)
        //{
        //    continue;
        //}
        //(void)nrf_serial_write(&serial0_uarte, &c, sizeof(c), NULL, 0);
        //(void)nrf_serial_flush(&serial0_uarte, 0);

        //ret = nrf_serial_read(&serial1_uarte, &c, sizeof(c), NULL, 1000);
        //if (ret != NRF_SUCCESS)
        //{
        //    continue;
        //}
        //(void)nrf_serial_write(&serial1_uarte, &c, sizeof(c), NULL, 0);
        //(void)nrf_serial_flush(&serial1_uarte, 0);
        
        nrf_delay_ms(1000);

    }
}

/** @} */




//*************************************************************************************************************
void GSM_init(void)
{
 APP_ERROR_CHECK(nrf_serial_init(&serial0_uarte, &m_uarte0_drv_config, &serial0_config));
}

void GSM_uninit(void)
{
APP_ERROR_CHECK(nrf_serial_uninit(&serial0_uarte));
}
ret_code_t GSM_send(void const * p_dataa, uint8_t len, uint32_t timeout_ms)//timeout in milliseconds
{
  //NRF_LOG_INFO("In GSM_send -----");
  //NRF_LOG_INFO("pdata: %s",p_dataa);
  //NRF_LOG_INFO("len: %u",len);
  return nrf_serial_write(&serial0_uarte, p_dataa, len, &written_size, timeout_ms);
}

ret_code_t GSM_tx_flush(uint32_t timeout_ms)//flushing tx queue
{
  return (nrf_serial_flush(&serial0_uarte, timeout_ms));
}

ret_code_t GSM_read(char * p_dataa, uint8_t len, uint32_t timeout_ms)
{
  return(nrf_serial_read(&serial0_uarte, p_dataa, len, &readed_size, timeout_ms));
}

ret_code_t GSM_abort_tx(void) //also clears the TX FIFO
{
  return (nrf_serial_tx_abort(&serial0_uarte));
}

ret_code_t GSM_drain_rx(void) //also Clears RX FIFO
{
  return (nrf_serial_rx_drain(&serial0_uarte));
}

void GSM_interrupt_enable(void)
{
        NVIC_EnableIRQ(UARTE1_IRQn);
}

void GSM_interrupt_disable(void)
{
	NVIC_DisableIRQ(UARTE1_IRQn);
}

void GSM_InterruptHandler(nrf_serial_t const *p_serial, nrf_serial_event_t event)
{
static uint8_t index1=0;
char bb;
switch (event)
  {
    case NRF_SERIAL_EVENT_RX_DATA:
      nrf_serial_read(&serial0_uarte, &bb, sizeof(bb), NULL, 50);
     // nrf_serial_read(&serial0_uarte, &bb, sizeof(bb), NULL, 2000);
      NRF_LOG_INFO("bb - %c",bb);
      rx_buff[rx_index++] = bb;
      //rx_len = strlen(rx_buff);
      //gsm_d.RxBuf[rx_index++] = bb;
      //gsm_d.RxBuf_Len = strlen(gsm_d.RxBuf);
      //rx_done++;
      }
       bool k=0;
      break;
    case NRF_SERIAL_EVENT_TX_DONE:
      //tx_done = true;
      break;
    case NRF_SERIAL_EVENT_DRV_ERR:
      //APP_ERROR_CHECK(nrf_serial_uninit(&serial0_uarte));
      //APP_ERROR_CHECK(nrf_serial_init(&serial0_uarte, &m_uarte1_drv_config, &serial0_config));
      break;
    case NRF_SERIAL_EVENT_FIFO_ERR:
      break;
    default:
      break;
  }
}
uint16_t kk=0;
  char * tt = 0;
void mqtt_connect(void)
{
  //char tx_buff[128];
  memset(&gsm_d,0,sizeof(gsm_d));

  GSM_tx_flush(500);
  GSM_abort_tx();
  written_size = 0;
  readed_size=0;
  sprintf(gsm_d.TxBuf,"AT+CRESET\r");

 // GSM_interrupt_enable();
 //memset(&gsm_d,0,sizeof(gsm_d));

  APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  while(1)
  {
    //memcpy(gsm_d.RxBuf,rx_buff,rx_len);
  char * tt1 = 0;
  char * tt2 = 0;
    if(tt1 = strstr((const char*)gsm_d.RxBuf,"ERR"))
    {
      memset(&gsm_d.RxBuf,0,sizeof(gsm_d.RxBuf));
      APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000)); 
      //memset(rx_buff,0,sizeof(rx_buff));
      //rx_len=0;
    }
    if(tt2 = strstr((const char*)gsm_d.RxBuf,"READY"))
    {
      //memset(rx_buff,0,sizeof(rx_buff));
      //rx_len=0;
      break; 
    }
    nrf_delay_ms(10);
  }
  GSM_tx_flush(500);
  GSM_abort_tx();

/*
//////////////////////////////////////////////////
  memset(&gsm_d,0,sizeof(gsm_d));
  tt = 0;
  sprintf(gsm_d.TxBuf,"AT\r");
  APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  while(tt == 0)
  {
    tt = strstr((const char*)gsm_d.RxBuf,"OK");
    nrf_delay_ms(10);
    APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  }
  GSM_tx_flush(500);
  GSM_abort_tx();

//////////////////////////////////////////////////////////////
  memset(&gsm_d,0,sizeof(gsm_d));
  tt = 0;
  sprintf(gsm_d.TxBuf,"ATE0\r");
  APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  while(tt == 0)
  {
    tt = strstr((const char*)gsm_d.RxBuf,"OK");
    nrf_delay_ms(10);
    APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  }
  GSM_tx_flush(500);
  GSM_abort_tx();

  //////////////////////////////////////////////////////////
  memset(&gsm_d,0,sizeof(gsm_d));
  tt = 0;
  sprintf(gsm_d.TxBuf,"AT+CMGF=1\r");
  APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  while(tt == 0)
  {
    nrf_delay_ms(1000);
    tt = strstr((const char*)gsm_d.RxBuf,"OK");
    APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  }
  GSM_tx_flush(500);
  GSM_abort_tx();

  nrf_delay_ms(2000);

  //////////////////////////////////////////////////////////
  memset(&gsm_d,0,sizeof(gsm_d)); 
  tt = 0;
  sprintf(gsm_d.TxBuf,"zzAT+CMGS=\"9652490222\"\r");
  gsm_d.TxBuf[0]=0x0d;
  gsm_d.TxBuf[1]=0x1a;
  //NRF_LOG_INFO("TxBuf: %s",gsm_d.TxBuf);
  //gsm_d.TxBuf[strlen(gsm_d.TxBuf)] = 0x1a;
  //gsm_d.TxBuf[strlen(gsm_d.TxBuf)+1] = '\0';
  //NRF_LOG_INFO("MAX LEN: %d",strlen(gsm_d.TxBuf));
  //for(int k=0; k<=strlen(gsm_d.TxBuf); k++)
  //{
  //NRF_LOG_INFO("gsm_d.TxBuf[%d]: %c",k,gsm_d.TxBuf[k]);
  //}
  APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  
  /*
  while(1)
  {    
    //NRF_LOG_INFO("RX MAX LEN: %d",strlen(gsm_d.RxBuf));
    //NRF_LOG_INFO("RX Index: %d",gsm_d.rx_index);
    nrf_delay_ms(2000);

    for( kk=0; kk<256; kk++)
    {
      if(gsm_d.RxBuf[kk] == 0x3e)
      {
        //tt = &gsm_d.RxBuf[kk];
        break;
      }
    }

    //APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  }
  */
  /*
  GSM_tx_flush(500);
  GSM_abort_tx();

  nrf_delay_ms(2000);
//////////////////////////////////////////////////////////////
  memset(&gsm_d,0,sizeof(gsm_d)); 
  tt = 0;
  sprintf(gsm_d.TxBuf,"Test Message-Raj");
  //NRF_LOG_INFO("MAX LEN: %d",strlen(gsm_d.TxBuf));
  gsm_d.TxBuf[strlen(gsm_d.TxBuf)] = 0x1a;
  //NRF_LOG_INFO("MAX LEN: %d",strlen(gsm_d.TxBuf));
  APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  while(tt == 0)
  {
    nrf_delay_ms(2000);
    tt = strstr((const char*)gsm_d.RxBuf,"CMGS");
    APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,strlen(gsm_d.TxBuf),1000));  
  }
  GSM_tx_flush(500);
  GSM_abort_tx();

  ////////////////////////////////////////////////
  */

  //GSM_interrupt_disable();
}//mqtt_connect
void off_GSM_interrupt(void)
{
  APP_ERROR_CHECK(nrf_serial_uninit(&serial0_uarte));
  APP_ERROR_CHECK(nrf_serial_init(&serial0_uarte, &m_uarte0_drv_config, &serial2_config));
}
void on_GSM_interrupt(void)
{
  APP_ERROR_CHECK(nrf_serial_uninit(&serial0_uarte));
  APP_ERROR_CHECK(nrf_serial_init(&serial0_uarte, &m_uarte0_drv_config, &serial0_config));
}
//**************************************************************************************************************

unsigned char sendCommand_resp(char* cmd_buf,unsigned int cmd_buf_len, char* resbuf, unsigned int respbuflen, int timeout)
{
  unsigned char status=0;
  unsigned char Tempbuff[GSM_MAX_CHARS]={0x00};
  
  memset(&gsm_d, 0x00, sizeof(gsm_d));
  memset(&rx_buff, 0x00, sizeof(rx_buff));
  rx_index=0;
  memset(Tempbuff,0x00,sizeof(Tempbuff));

  //GSM_Received = 0;
  //GSM_Receive_count = 0;
  //rx_done = 0;

  memcpy(gsm_d.TxBuf,cmd_buf,cmd_buf_len);
  gsm_d.TxBuf_Len=cmd_buf_len;
  flushuart();

 //GSM_interrupt_enable();

  APP_ERROR_CHECK(GSM_send(gsm_d.TxBuf,gsm_d.TxBuf_Len,1000));

  while(timeout--)
  {
    char * tt=0;
    char * tt1=0;
    if(rx_index!=0){
    memcpy(gsm_d.RxBuf,rx_buff,rx_index);
    NRF_LOG_INFO("gsm_d.RxBuf: %s",gsm_d.RxBuf);
    NRF_LOG_INFO("rx_buff: %s",rx_buff);
    }

    //tt = strstr((const char*)rx_buff,resbuf);
    //tt1 = strstr((const char*)rx_buff,"ERR");
    tt = strstr((const char*)gsm_d.RxBuf,resbuf);
    tt1 = strstr((const char*)gsm_d.RxBuf,"ERR");
    if(tt1 !=0)
    { 
      //memset(rx_buff,0,sizeof(rx_buff));
      //rx_len=0;
      //status = 0;
      //return status;
    }
    if(tt != 0)
    {
      //memset(rx_buff,0,sizeof(rx_buff));
      //rx_len=0;
      status = 1;
      return status;
    }
    nrf_delay_ms(1);
  }
  return status;
}

void flushuart(void)
{
  GSM_tx_flush(500);
  GSM_abort_tx();
  GSM_drain_rx();
}
uint8_t retry=0;
//uint8_t send_sms(char* num, char* msg, unsigned int msg_length)
//{
//  uint8_t status = 0;
//  //uint8_t retry=0;
//  uint8_t cntrl=0;
//  char l_buf[128];
      
//  strt:

//  if(cntrl)
//  {
//    if((!sendCommand_resp("AT+CRESET\r",strlen("AT+CRESET\r"),"OK",strlen("OK"),5000))&&(retry<10))
//    {
//      cntrl = 1;
//      retry++;
//      goto strt;
//    }
//    else{
//    NRF_LOG_INFO("JUST WAITING FOR 13 SECS");
//      nrf_delay_ms(13000);
//    }
//  }

//  if((!sendCommand_resp("AT\r",strlen("AT\r"),"OK",strlen("OK"),5000))&&(retry<10))
//    {
//      cntrl = 1;
//      retry++;
//      goto strt;
//    }

//  if((!sendCommand_resp("ATE0\r",strlen("ATE0\r"),"OK",strlen("OK"),5000))&&(retry<10))
//    {
//      cntrl = 1;
//      retry++;
//      goto strt;
//    }

//  if((!sendCommand_resp("AT+CMGF=1\r",strlen("AT+CMGF=1\r"),"OK",strlen("OK"),5000))&&(retry<10))
//    {
//      cntrl = 1;
//      retry++;
//      goto strt;
//    }
    
//    memset(l_buf,0,sizeof(l_buf));
//    sprintf(l_buf,"AT+CMGS=\"%s\"\r",num);
//  sendCommand_resp(l_buf,strlen(l_buf),(char*)0x3e,1,5000);  

//     memset(l_buf,0,sizeof(l_buf));
//     memcpy(l_buf,msg,msg_length);
//     l_buf[strlen(l_buf)] = 0x1a;
//  if((!sendCommand_resp(l_buf,strlen(l_buf),"CMGS",strlen("CMGS"),5000))&&(retry<10))
//    {
//      cntrl = 1;
//      retry++;
//      goto strt;
//    }
//if(retry >= 10)
//{
//  return 0;
//}
//return 1;
    
//}//send_sms

uint8_t mqtt_st=0;
uint8_t mqtt_publish(void)
{
  uint8_t status = 0;
   mqtt_st=0;
  //uint8_t 
  retry=0;
  uint8_t cntrl = 0;
  char l_buf[128];
  uint8_t is_start=0;
  uint8_t is_stop=0;
      
  strt:

  //if(cntrl)
  //{
  //  if((!sendCommand_resp("AT+CRESET\r",strlen("AT+CRESET\r"),"OK",strlen("OK"),5000))&&(retry<10))
  //  {
  //    cntrl = 1;
  //    retry++;
  //    goto strt;
  //  }
  //  else{
  //    resett = true;
  //  }
  //}
  //if(resett)
  //{
  //  nrf_delay_ms(25000);
  //}

  mqtt_st=0;
  ATE0:
  if((!sendCommand_resp("ATE0\r",strlen("ATE0\r"),"OK",strlen("OK"),5000))&&(retry<10))
    {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto ATE0;
      }
      cntrl = 1;
      retry++;
      goto strt;
    }

  mqtt_st=0;
  AT:
  if((!sendCommand_resp("AT\r",strlen("AT\r"),"OK",strlen("OK"),5000))&&(retry<10))
    {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto AT;
      }
      cntrl = 1;
      retry++;
      goto strt;
    }


if(is_start){
//    mqtt_st=0;
//  CMQTTREL:
  
  //if((!sendCommand_resp("AT+CMQTTREL=1\r",strlen("AT+CMQTTREL=1\r"),"OK",strlen("OK"),5000))&&(retry<10))
  //  {
  //    mqtt_st++;
  //    if(mqtt_st<3)
  //    { 
  //      goto CMQTTREL;
  //    }
  //    //cntrl = 1;
  //    //retry++;
  //    //goto strt;
  //  }

  mqtt_st = 0;
  MQTT_STOP:

  if((!sendCommand_resp("AT+CMQTTSTOP\r",strlen("AT+CMQTTSTOP\r"),"CMQT",strlen("CMQT"),5000))&&(retry<10))
    {
      mqtt_st++;
      memset(&gsm_d, 0x00, sizeof(gsm_d));
      memset(&rx_buff, 0x00, sizeof(rx_buff));
      rx_index=0;
      
      if(mqtt_st<3)
      { 
        goto MQTT_STOP;
      }
      cntrl = 1;
      retry++;
      goto strt;
    }
    else{
      is_stop=1;
      is_start = 0;
    }
 }


if(!is_start){
    mqtt_st=0;
  MQTT_START:
  //if((!sendCommand_resp("AT+CMQTTSTART\r",strlen("AT+CMQTTSTART\r"),"OK",strlen("OK"),5000))&&(retry<10))
  if((!sendCommand_resp("AT+CMQTTSTART\r",strlen("AT+CMQTTSTART\r"),"+CMQTTSTART",strlen("+CMQTTSTART"),5000))&&(retry<10))
    {
      
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto MQTT_START;
      }
      if(is_stop)
      {
      cntrl = 1;
      retry++;
      goto strt;
      }
      mqtt_st = 0;
      goto MQTT_STOP;
    }
    else{
      is_start = 1;
      is_stop = 0;
    }
}
    mqtt_st=0;
  CMQTTACCQ:
  if((!sendCommand_resp("AT+CMQTTACCQ=1,\"QWERTY\",0\r",strlen("AT+CMQTTACCQ=1,\"QWERTY\",0\r"),"OK",strlen("OK"),5000))&&(retry<10))
    {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto CMQTTACCQ;
      }
      cntrl = 1;
      retry++;
      goto strt;
    }

    mqtt_st=0;
  CMQTTWILL:
  if(!sendCommand_resp("AT+CMQTTWILLMSG=1,4,0\r",strlen("AT+CMQTTWILLMSG=1,4,0\r"),&arraow,1,5000))
  {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto CMQTTWILL;
      }
      cntrl = 1;
      retry++;
      goto strt;
  }

    mqtt_st=0;
  D1:

      memset(l_buf,0,sizeof(l_buf));
      sprintf(l_buf,"DISC");      
     //l_buf[strlen(l_buf)] = 0x1a;
  if((!sendCommand_resp(l_buf,strlen(l_buf),"OK",strlen("OK"),5000))&&(retry<10))
    {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto D1;
      }
      cntrl = 1;
      retry++;
      goto strt;
    }


    mqtt_st=0;
  CMQTTCONNECT:
//   if((!sendCommand_resp("AT+CMQTTCONNECT=1,\"tcp://io.adafruit.com:1883\",120,1,\"himanc2\",\"aio_xFOr446u5ZPiSP9GbuLLu3Ee4LC3\"\r",strlen("AT+CMQTTCONNECT=1,\"tcp://io.adafruit.com:1883\",120,1,\"himanc2\",\"aio_xFOr446u5ZPiSP9GbuLLu3Ee4LC3\"\r"),"OK",strlen("OK"),5000))&&(retry<10))
   if((!sendCommand_resp("AT+CMQTTCONNECT=1,\"tcp://io.adafruit.com:1883\",120,1,\"himanc2\",\"aio_xFOr446u5ZPiSP9GbuLLu3Ee4LC3\"\r",strlen("AT+CMQTTCONNECT=1,\"tcp://io.adafruit.com:1883\",120,1,\"himanc2\",\"aio_xFOr446u5ZPiSP9GbuLLu3Ee4LC3\"\r"),"CMQTTCONNECT",strlen("CMQTTCONNECT"),5000))&&(retry<10))
    {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto CMQTTCONNECT;
      }
      cntrl = 1;
      retry++;
      goto strt;
    }

//nrf_delay_ms(10000);
    mqtt_st=0;
  CMQTTTOPIC:
  if(!sendCommand_resp("AT+CMQTTTOPIC=1,26\r",strlen("AT+CMQTTTOPIC=1,26\r"),&arraow,1,15000))
  {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto CMQTTTOPIC;
      }
      cntrl = 1;
      retry++;
      goto strt;
  }

    mqtt_st=0;
  D2:
      memset(l_buf,0,sizeof(l_buf));
      sprintf(l_buf,"himanc2/feeds/welcome-feed"); 
     //l_buf[strlen(l_buf)] = 0x1a;
  if((!sendCommand_resp(l_buf,strlen(l_buf),"OK",strlen("OK"),5000))&&(retry<10))
    {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto D2;
      }
      cntrl = 1;
      retry++;
      goto strt;
    }

    mqtt_st=0;
  CMQTTPAYLOAD:
  if(!sendCommand_resp("AT+CMQTTPAYLOAD=1,2\r",strlen("AT+CMQTTPAYLOAD=1,2\r"),&arraow,1,5000))
  {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto CMQTTPAYLOAD;
      }
      cntrl = 1;
      retry++;
      goto strt;
  }

    mqtt_st=0;
  D3:
      memset(l_buf,0,sizeof(l_buf));
      sprintf(l_buf,"69"); 
     //l_buf[strlen(l_buf)] = 0x1a;
  if((!sendCommand_resp(l_buf,strlen(l_buf),"OK",strlen("OK"),5000))&&(retry<10))
    {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto D3;
      }
      cntrl = 1;
      retry++;
      goto strt;
    }
 

     mqtt_st=0;
  CMQTTPUB:
  if(!sendCommand_resp("AT+CMQTTPUB=1,0,120\r",strlen("AT+CMQTTPUB=1,0,120\r"),"OK",strlen("OK"),5000))
  {
      mqtt_st++;
      if(mqtt_st<3)
      { 
        goto CMQTTPUB;
      }
      cntrl = 1;
      retry++;
      goto strt;
  }
   
if(retry >= 10)
{
  return 0;
}
return 1;
    
}//mqtt publish